/*
 *========================================================================
 * $Id: directory.c 30 2004-01-25 22:38:21Z rgb $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

#include "flashcard.h"


/* 
 *==================================================================
 * This file contains routines for displaying and changing
 * directories and for selecting directories or flashfiles for
 * display.  It works in the directory box dirb, which exactly
 * overlaps with historyb.  This box is brought to the top "inside"
 * these routines, and the history box/panel is brought to the
 * top on exit.
 *==================================================================
 */

/*
 * This routine checks for single character answers and does the
 * relevant menu choices (if possible).
 */
void change_directory()
{

 char newdir[1024];
 struct stat statbuf;

 display_flashdir();

 newdir[0] = 1;
 if(timertid != 0) pthread_cancel(timertid);
 while(newdir[0] != 0){
   mvwprintw(answerb.win,0,0,"Enter new directory name");
   mvwprintw(answerb.win,1,0,"or enter to return-> ");
   wclrtoeol(answerb.win);
   wrefresh(answerb.win);
   snoop_answer();
   wgetnstr(answerb.win,newdir,K);
   wclear(answerb.win);
   if(verbose){
     wprintw(dirb.win,"Trying to stat/display %s\n",newdir);
     wrefresh(dirb.win);
   }

   /* Empty directory name returns */
   if(newdir[0] == 0) {
     show_panel(historyb.pan);
     return;
   }
   
   /*
    * We check to see if the new directory name stats.
    */
   if(stat(newdir,&statbuf) == 0) {
     /* Yes, apparently it does.  So we accept it. */
     strncpy(state.flashdir,newdir,1024);
     chdir(state.flashdir);
     wprintw(dirb.win,"New directory:\n");
     wrefresh(dirb.win);
     display_flashdir();
   } else {
     wprintw(dirb.win,"-----------------------------------------------------------------\n");
     wprintw(dirb.win,"Error!  Directory %s does not stat!\n",newdir);
     wprintw(dirb.win,"Check spelling of directory and try again.\n");
     wrefresh(dirb.win);
   }

 }

}

display_flashdir()
{

 int i,n,nfiles,x,y;
 struct dirent **namelist;

 /*
  * Display the contents of current working directory
  */
 getcwd(state.flashdir,1024);
 nfiles = scandir(state.flashdir, &namelist, 0, alphasort);
 wclear(dirb.win);
 wprintw(dirb.win,"-----------------------------------------------------------------\n");
 wprintw(dirb.win,"%s\n",state.flashdir);
 wrefresh(dirb.win);
 i = 0;
 n = 0;
 if (nfiles >= 0){
   while(i < nfiles) {
     if(n == 0){
       wprintw(dirb.win,"%s", namelist[i]->d_name);
     } else {
       wprintw(dirb.win," %s", namelist[i]->d_name);
     }
     if(n < 2){
       getyx(dirb.win,x,y);
       mvwprintw(dirb.win,x,(n+1)*24,"");
       n++;
     } else {
       n = 0;
       wprintw(dirb.win,"\n");
     }
     free(namelist[i]);
     i++;
   }
   free(namelist);
 }
 wprintw(dirb.win,"\n");
 wrefresh(dirb.win);
 show_panel(dirb.pan);

}
 
