/*
 *========================================================================
 * $Id: file.c 31 2004-01-25 23:28:59Z rgb $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

#include "flashcard.h"

void get_flashfile()
{

 int delay=0;

 /*
  * Stop the timer on the current problem if there is one
  */
 if(timertid != 0) pthread_cancel(timertid);
 echo();
 display_flashdir();
 wclear(answerb.win);
 mvwprintw(answerb.win,0,0,"Enter new flash file name");
 mvwprintw(answerb.win,1,0,"Or enter to return -> ");
 wrefresh(answerb.win);
 snoop_answer();
 wgetnstr(answerb.win,state.flashfile,1024);

 /*
  * There is a flashfile name (which might be null) at this point.  Now to
  * set state.
  */
 if(verbose){
   wprintw(dirb.win,"get_flashfile(): got flashfile name %s\n",state.flashfile);
   wrefresh(dirb.win);
   sleep(delay);
 }
   /*
    * While state.state is NONE we need to get a valid file name or we can
    * do nothing.  If the filename entered above is non-null, we set
    * state.state to NEW and go on to check if it is indeed a file.  If
    * the filename entered is null (empty) AND state.suspend is YES we're
    * running and can just resume, so we set state.state to RUN.  If
    * state.suspend is NO, though, we cannot resume running and have no
    * file, so we set state.state to NONE to force a return to this
    * routine.
    */
 if(state.flashfile[0] != 0){
   state.state = NEW;
 } else {
   if(state.suspend == YES){
     strncpy(state.flashfile,flashfile_save,LONGBUF);
     state.state = RUN;  /* resumes old run */
   } else {
     state.state = NONE;
   }
 }

}


void stat_flashfile()
{

 /* To help decide if flashfile is goot */
 struct stat statbuf;

 /*
  * If state.state is NEW, we have a non-null flashfile name, so we try
  * to stat it to see if it is a file.  If it doesn't stat we issue an
  * error message first.  Then if state.suspend is NO, we set
  * state.state to NONE to force another try at loading a valid file.
  * If state.suspend is YES, we sigh and resume running (the user
  * can always try again).  If the file DOES stat, we set state.state
  * to LOAD, the next step.
  */
 if(stat(state.flashfile,&statbuf)){
   /* Nope no flashfile here boss */
   wprintw(historyb.win,"Error opening %s in:\n",state.flashfile);
   wrefresh(historyb.win);
   if(state.suspend == NO){
     state.state = NONE; /* return to get another file */
   } else {
     state.state = RUN; /* resume running existing flashfile */
   }
 } else {
   state.state = LOAD;
 }

}

void run_flashfile()
{

 int i,cnt,cntmod,delay=0;

 /*
  * Finally, if state.state is RUN (or RERUN), a flashfile exists and is
  * parsed into problems.  If state.suspend is NO, we initialize the
  * problem set according to the various parameters set in state.  If
  * state.suspend is YES, we are resuming an existing computation and the
  * global (effectively static) runstate variables should know where we
  * are.  We skip initializing/altering them and just reenter the main
  * loop precisely where we left it (resetting state.suspend back to NO
  * along the way).
  */

 /*
  * Shuffling has to be a global decision now, independent of problem
  * type as we want to be able to present mixed flashcard files with
  * more than one type of problem.  We therefore set presentation_type
  * (which can be r(andom) or l(ist)) according to:
  *  default r
  *  otherwise set from flashfile
  *  override from command line
  * 
  * In list mode, we just cycle through the list repeatedly until we
  * have cumulated number_of_problems problems.  Else we shuffle the indices 
  * of the problem list whenever cnt is a multiple of the number_of_problems
  * (including zero, which handles the most common case where the number
  * of problems in the list determines num_probs and each problem is
  * presented exactly once.  We quit when we've done enough problems in
  * test mode, loop "forever" in practice (non-test) mode.
  */
 if(verbose){
   wprintw(historyb.win,"run_flashfile():\n");
   if(state.presentation == RANDOM) {
     wprintw(historyb.win,"About to present %d randomly shuffled problems.\n",number_of_problems);
   } else {
     wprintw(historyb.win,"About to present %d problems in list order.\n",number_of_problems);
   }
   wrefresh(historyb.win);
   sleep(delay);
 }

 /*
  * This effectively clears/updates the score boxes.
  */
 show_score();
 
 cnt = 0;
 while(-1) {  /* loop forever, break out on a condition. */
   /*
    * We reshuffle after running through each list of problems once.
    * shuffle() only shuffles in random mode.  Otherwise it just fills
    * iorder[cntmod] with the ordered list indices.
    */
   cntmod = cnt%number_of_problems;
   if(cntmod == 0) {
     if(verbose){
       wprintw(historyb.win,"run_flashfile():\n");
       wprintw(historyb.win,"Shuffling on cntmod %d\n",cntmod);
       wrefresh(historyb.win);
       sleep(delay);
     }
     shuffle();
   }
   if(verbose){
     wprintw(historyb.win,"run_flashfile():\n");
     wprintw(historyb.win,"Presenting problem %d\n",iorder[cntmod]);
     wrefresh(historyb.win);
     sleep(delay);
   }
   present(iorder[cntmod]);
   if(state.suspend == YES) return;
   /*
    * If we got it wrong and are in practice mode, we might need to
    * present this state.repeat times!
    */
   if(pstate.repeat_count == 0){
     cnt++;
   }
   if(verbose){
     wprintw(historyb.win,"run_flashfile():\n");
     wprintw(historyb.win,"incremented cnt = %d\n",cnt);
     wrefresh(historyb.win);
     sleep(delay);
   }

   if(cnt >= state.num_probs && state.num_probs > 0) {
     state.state = END;
     break;
   }

 }

}


void end_flashfile()
{

 int i,cnt,cntmod,delay=0;

 /*
  * If state.state is END, we have just parsed a valid new flashfile
  * BUT we also have a suspended flashfile session on hand.  So we
  * end the suspended session gracefully.  This had better not be
  * ABLE to fail, and on success it should set state.state to RUN.
  */

 if(verbose){
   wprintw(historyb.win,"\nEnding current flashcard file.\n");
   wrefresh(historyb.win);
   sleep(delay);
 }

 /*
  * Pause for just long enough to appreciate that last answer.
  */
 wprintw(historyb.win,"\n");
 wprintw(historyb.win,"=============================================================================\n");
 wprintw(historyb.win,"                          Finished %s\n",flashfile_save);
 wprintw(historyb.win,"Fabulous!  You completed  %d  problems in",state.count);
 if(state.mode == TEST) {
   wprintw(historyb.win," test ");
 } else {
   wprintw(historyb.win," practice ");
 }
 wprintw(historyb.win,"mode.\n");
 wprintw(historyb.win,"Check out your score below and press Enter to continue.\n");
 wprintw(historyb.win,"\n");
 wprintw(historyb.win,"=============================================================================\n");
 wrefresh(historyb.win);
 wclear(answerb.win);
 mvwprintw(answerb.win,1,0,"Press Enter to continue -->");
 wgetnstr(answerb.win,answer,64);

 if(verbose){
   wprintw(historyb.win,"Resetting counters.  Canceling threads.\n");
   wrefresh(historyb.win);
   sleep(delay);
 }

 /*
  * Reset state.  We've just ended a problem set, so we set state.state
  * to RUN and zero all the counters.  Note that we turn off
  * state.suspend here if we haven't already.
  */
 state.state = RUN;
 state.suspend = NO;
 state.correct = 0;
 state.incorrect = 0;
 state.count = 0;
 state.total_time = 0;
 pthread_cancel(timertid);
 pthread_cancel(probtid);

}

