/*
 *========================================================================
 * $Id: flashcard.h 35 2006-12-20 04:22:18Z rgb $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

#include "copyright.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include <curses.h>
#include <panel.h>
#include <menu.h>
#include <pthread.h>
#include <sys/stat.h>
#include <dirent.h>
/* the main gnome include */
/* #include <gnome.h> */

/* libxml includes */
/*
 * COMPAT using xml-config --cflags to get the include path this will
 * work with both
 */
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

/*
 *========================================================================
 * Useful defines
 *========================================================================
 */

#define STDIN	stdin
#define STDOUT	stdout
#define STDERR	stderr
#define YES	1
#define NO	0
#define PI 3.141592653589793238462643
#define K 1024
#define K2 512
#define K4 256
#define PAGEBUF 4096
#define LONGBUF 1024
#define LINEBUF 80
#define TYPEBUF 32
#define SHOWMAX 10
#define HOT 1
#define LINE 0

/*
 * This enumerates the various modes of operation as we develop them for
 * use in case switches and the like.  These are all mutually exclusive.
 */
 typedef enum {
   PRACTICE,
   TEST,
   LEARNING,
   NMODES
 } Modes;

 /*
  * These mean:
  *  NONE = no flashfile defined in state.flashfile at all
  *  NEW = state.flashfile contains non-null string, file not validated
  *  LOAD = state.flashfile exists and has been XML parsed successfully
  *  RUN = state.flashfile is being run
  *  END = state.flashfile is finished and its results should be displayed
  *  SUSPEND = state.flashfile is suspended while e.g. a menu is displayed
  */
 typedef enum {
   NONE,
   NEW,
   LOAD,
   RUN,
   END,
   SUSPEND,
   NSTATES
 } States;

 typedef enum {
   UNSET,
   LIST,
   RANDOM,
   NPRESENTATIONS
 } Presentation;

 /*
  * Have to define Box here so I can use it in prototypes.
  * Probably want to put all struct definitions in struct.h,
  * maybe all enums in enum.h
  */
 typedef struct {
   WINDOW *frame;   /* Frame box */
   WINDOW *win;     /* Window inside frame */
   PANEL *pan;      /* Panel associated with window */
   int numrows;     /* Number of lines in window */
   int numcols;     /* Number of columns in window */
   int screenrow;   /* top left corner, screen coords (range 0-(LINES-1)) */
   int screencol;   /* top left corner, screen coords (range 0-(COLS-1)) */
   int showrow;     /* top row displayed if pad in pad coords */
   int showcol;     /* top col displayed if pad in pad coords */
 } Box;

 /*
  *========================================================================
  * Subroutine Prototypes
  *========================================================================
  */
 void Usage();
 /* file.c */
 void get_flashfile();
 void stat_flashfile();
 void run_flashfile();
 void end_flashfile();
 /* grade_answer.c */
 void get_answer(int *pi);
 void grade_generic(int *pi);
 void grade_answer(int *pi);
 void grade_arithmetic(int *pi);
 void grade_multiple_choice(int *pi);
 void grade_spelling(int *pi);
 void grade_true_false(int *pi);
 void increment_correct();
 void increment_incorrect();
 void show_score();
 /* parse.c */
 int parse(char *inbuffer,char **outfields,int maxfields,int maxfieldlength);
 /* parsexml_flashfile.c */
 void parsexml_flashfile();
 void parse_problem(xmlNodePtr problem,int i);
 void fix_lf(char *buf,int buflen);
 /* present.c */
 void present(int pi);
 int present_problem(int pi);
 /* present_utils.c */
 void show_problem(int *pi);
 int present_question(int pi);
 void present_choices(int pi);
 void present_prompt(char *prompt);
 void play_word(int pi);
 void play_audio(int pi);
 /* screen.c */
 void start_screen();
 void makebox(Box *box);
 /* shuffle.c */
 void shufflelist(int *ilist, int imax);
 void shuffle();
 /* timer.c */
 void timer(int *pi);
 /* snoop_answer.c */
 char snoop_answer();
 
 /*
  *========================================================================
  * Global Variables
  *
  * Set the default values and override from the command line in parsecl().
  *========================================================================
  */
 int verbose;
 /* flags -- now cruft (all in state structs or unneeded)
 int quit_flag,repeat_flag,done_flag,flashfile_flag,correctflag;
 */
/* control variables -- now cruft (all in state structs)
 int mode,time_max,num_probs,repeat,answermode;
  */

 /* counters  -- now cruft (all in state structs)
 int correct,incorrect,count,maximum_time,play_time,problem_time,total_time;
  */

 /* the answer (also interactive control!) */
 char answer[LINEBUF],saveanswer[LINEBUF];
 /* pthread thread ids for control purposes */
 pthread_t probtid,timertid,playtid;

 /*
  * The following struct should be the general purpose "problem" struct.
  * It must be filled in dynamically as we encounter each problem type.
  * It is probably worthwhile to parse each problem type with its own
  * unique code so we don't have to worry so much about customization.
  * We can think about reusing code fragments only when they wrap up
  * nicely in a subroutine.  Note that we have to define Choice before
  * Problem.
  */

 typedef struct {
   char flashfile[LONGBUF];
   char flashdir[LONGBUF];
   int state;
   int suspend;
   int sound;
   int mode;
   int time_max;
   int presentation;
   int num_probs;
   int repeat;
   int correct;
   int incorrect;
   int count;
   int total_time;
 } State;

 State state;

 /*
  * Problem state struct
  */
 typedef struct {
   int correct;
   int problem_time;
   int repeat_count;
 } Pstate;

 Pstate pstate;

 /* Full width boxes */
 Box headerb,historyb,dirb,answerb,menub,footerb;
 /* labelled score/time boxes */
 Box rightb,wrongb,rightlb,wronglb;
 Box pctb,pctlb,elapsedb,elapsedlb,timerb,timerlb;

 WINDOW *flashwin;  /* main window/frame */

 typedef struct {
   /* "answer" id, shared aid's are "or", distinct are "and" */
   char *aid;
   char *text;
 } Choice;

 typedef struct {
   char type[TYPEBUF];
   char question[LONGBUF];
   char audio[LONGBUF];
   char answer[LONGBUF];
   char prompt[LINEBUF];
   int show;
   int numchoices;
   Choice *choices;
   char explanation[PAGEBUF];
 } Problem;

 Problem *problem_list;

 /*
  * We may find a use for a place to save old directory names.
  * We'll definitely need a place to save the current file name
  * while we try stat'ing new ones.
  */
 char flashdir_save[LONGBUF];
 char flashfile_save[LONGBUF];

 /*
  * Scratch space to hold the shuffled presentation lists and controls
  * thereof.  Note that we can have LOTS of choices including foolers
  * in multichoice, but only SHOWMAX can be presented at a time, including
  * all the answers.
  */
 int choicelist[K];
 char **aidlist,**answerlist;
 int number_of_aids,number_of_answers,number_of_problems,*iorder;

 /*
  * multichoice array of choice letters.
  */
 char **choiceletter;
