/*
 *========================================================================
 * $Id: flashcard_work.c 27 2004-01-25 06:35:11Z rgb $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

/*
 *========================================================================
 *  Project work template.
 *========================================================================
 */

#include "flashcard.h"

void flashcard_work()
{

 int delay=0;

 /*
  * Exit infinite loop only with Alt-Q(uit) or ^C.
  *
  * The work cycle in this program is:
  *   a) Zero counters for flashfile session.
  *   b) Obtain a flashfile name (if necessary)
  *   c) Load/parse the flashfile
  *   d) Run the flashfile
  *   e) "End" the flashfile (freeing all memory)
  *   f) Return to zero counters...
  *
  * This cycle is NOT quite right.  We can either have or not have
  * a flashfile defined by the end of startup.  I almost think that
  * we need to make a "state" struct that contains things like:
  *  flashfile (name)
  *  directory (name)
  *  state (NEW,LOAD,RUN,END,SUSPEND from enum)
  *  sound (boolean)
  *  mode (TEST,PRACTICE from enum)
  *  presentation (LIST,RANDOM from enum)
  *  time_max (maximum time per problem in seconds, test mode only)
  *  num_probs (number of problems presented, test mode only)
  *  ...other options controllable via command line or menu.
  *
  * With this in hand, we can easily plan to exit any flashfile
  * "cleanly" OR continue when e.g. we opt to change directory
  * or flashfile and possibly change our mind and want to continue.
  *
  * Note that end_flashfile should NOT count the current problem
  * wrong.  In test mode right/wrong should be out of num_probs
  * always.  In practice mode right/wrong should be out of
  * ANSWERED problems at the time of termination.
  *
  * Additional features:  We need to be able to suspend (Alt-S) and resume
  * (any key) even in test mode.  This tool is NOT to be used to actually
  * administer tests but to help students practice for them.  However,
  * it is clear that it will get more use with some carrot/stick action,
  * so we need to be able to create a whole "monetary" system (pun intended)
  * to permit a parent to "monitor" child usage and reward/punish as need
  * be.  This leads us to the notion of workminder.  See associated design
  * file...
  */
  
 while(-1){

   /*
    * While state.state is NONE we need to get a valid file name or we can
    * do nothing.  If the filename entered above is non-null, we set
    * state.state to NEW and go on to check if it is indeed a file.  If
    * the filename entered is null (empty) AND state.suspend is YES we're
    * running and can just resume, so we set state.state to RUN.  If
    * state.suspend is NO, though, we cannot resume running and have no
    * file, so we set state.state to NONE to force a return to this
    * routine.
    */
   if(state.state == NONE || state.state == SUSPEND){
     if(verbose){
       wprintw(historyb.win,"flashcard_work(): get_flashfile();\n");
       wrefresh(historyb.win);
       sleep(delay);
     }
     get_flashfile();
   }

   /*
    * If state.state is NEW, we have a non-null flashfile name, so we try
    * to stat it to see if it is a file.  If it doesn't stat we issue an
    * error message first.  Then if state.suspend is NO, we set
    * state.state to NONE to force another try at loading a valid file.
    * If state.suspend is YES, we sigh and resume running (the user
    * can always try again).  If the file DOES stat, we set state.state
    * to LOAD, the next step.
    */
   if(state.state == NEW){
     if(verbose){
       wprintw(historyb.win,"flashcard_work(): stat_flashfile();\n");
       wrefresh(historyb.win);
       sleep(delay);
     }
     stat_flashfile();
   }

   /*
    * If state.state is LOAD, the file stats, so it is time to actually
    * load it and parse it.  If loading fails BEFORE any records are parsed
    * (so the old problem_list is still valid) and
    * state.suspend is YES, state.state should go to RUN to resume.  If
    * parsing fails but the old problem list is corrupted, state.state
    * has to go back to NONE (with an Error Message!) because we can
    * no longer resume the old one.  If parsing succeeds, a new file
    * is loaded and the old problem set is no longer even there.  At that
    * point state.state goes to RUN if state.suspend is NO (so there is
    * no old session to terminate and wrap up) and END if state.suspend is
    * YES (so the old session can print out final statistics, interact
    * with any possible future workminder tool, etc.)
    */
   if(state.state == LOAD){
     if(verbose){
       wprintw(historyb.win,"flashcard_work(): parse_flashfile();\n");
       wrefresh(historyb.win);
       sleep(delay);
     }
     parsexml_flashfile();
   }

   /*
    * If state.state is END, we have just parsed a valid new flashfile
    * BUT we also have a suspended flashfile session on hand.  So we
    * end the suspended session gracefully.  This had better not be
    * ABLE to fail, and on success it should set state.state to RUN.
    */
   if(state.state == END){
     if(verbose){
       wprintw(historyb.win,"flashcard_work(): end_flashfile();\n");
       wrefresh(historyb.win);
       sleep(delay);
     }
     end_flashfile();
   }

   /*
    * Finally, if state.state is RUN (or RERUN), a flashfile exists and is
    * parsed into problems.  If state.suspend is NO, we initialize the
    * problem set according to the various parameters set in state.  If
    * state.suspend is YES, we are resuming an existing computation and the
    * global (effectively static) runstate variables should know where we
    * are.  We skip initializing/altering them and just reenter the main
    * loop precisely where we left it (resetting state.suspend back to NO
    * along the way).
    */
   if(state.state == RUN){
     if(verbose){
       wprintw(historyb.win,"flashcard_work(): run_flashfile();\n");
       wrefresh(historyb.win);
       sleep(delay);
     }
     run_flashfile();
   }
 }
   
}

