/*
 *========================================================================
 * $Id: parsecl.c 27 2004-01-25 06:35:11Z rgb $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

#include "flashcard.h"

/*
 * This routine sets certain globals to their default values and permits
 * those defaults to be overridden on the command line.
 */

void parsecl(int argc, char **argv)
{
 int opt_c,opt_errflg=0;
 extern char *optarg;
 extern int optind, opterr, optopt;


 /* 
  * Initialize global variables with defaults, then override if
  * desired on command line.  Read the getopt man page to see how
  * to set other variables.  Or insert your own parsing code in here --
  * Sometimes I use getopts, sometimes I parse myself, sometimes both.
  */
 verbose = 0;			/* Default is quiet */

 /*
  * Initialize state variable struct.  Some of these will get reset below
  * and in startup.  We start out with a default of no flashfile and the
  * current working directory.
  */
 strncpy(state.flashfile,"",LONGBUF);
 strncpy(state.flashdir,".",LONGBUF);
 state.state = NONE;          /* No flashfile, no state */
 state.suspend = NO;          /* Not suspended RUN state */
 state.sound = YES;           /* Sound on */
 state.mode = PRACTICE;       /* Practice mode */
 state.time_max = 0;          /* No time limit in practice mode */
 state.presentation = UNSET;  /* This permits the tag attribute to work */
 state.num_probs = 0;         /* No problem limit in practice mode */
 state.repeat = 1;            /* Repeat all problems one time on mistake */
 state.correct = 0;           /* No problems correct yet */
 state.incorrect = 0;         /* or incorrect */
 state.count = 0;             /* or count */
 state.total_time = 0;        /* or elapsed time */
 
 while ((opt_c = getopt(argc,argv,"d:f:hlm:n:rt:v")) != EOF){
   switch (opt_c){
     case 'd':
       strncpy(state.flashdir,optarg,LONGBUF);
       break;
     case 'f':
       strncpy(state.flashfile,optarg,LONGBUF);
       state.state = NEW;
       break;
     case 'h':
       Usage();
       exit(0);
       break;
     case 'l':
       state.presentation = LIST;
       break;
     case 'm':
       /*
        * Make sure the mode docs match the enumerated list in flashcard.h
	*/
       state.mode = atoi(optarg);
       break;
     case 'n':
       state.num_probs = atoi(optarg);
       break;
     case 'r':
       state.presentation = RANDOM;
       break;
     case 't':
       state.time_max = atoi(optarg);
       break;
     case 'v':
       verbose = 1;
       break;
     case '?':
       opt_errflg++;
   }
 }

 /*
  * num_probs default reset for test mode only.  Note that 0 means
  * "never stop" in practice mode, a number means "quit after this
  * number of correct answers".
  */
 if((state.mode == TEST) && (state.num_probs == 0)) state.num_probs = 100;

 /*
  * If there are no more arguments, do a sanity check and return, we are
  * done.
  */
 if(argc-optind == 0){
   return;
 }

 /*
  * Anything else left is an erroneous call and should cause the Usage
  * message to be printed.  memtest -h or --help will therefore "work".
  */

 Usage();

}

char usage[] =
"Usage:\n"
"  flashcard -f flashfile.flash [-l] [m mode_test] [-n num_probs] \\\n"
"            [-r] [-t time_max] [-h] [-v]\n"
"\n"	    
"    -l forces a list (in order) presentation, overriding presentation\n"
"       specified in .flash file or default.\n"
"    -m mode_test selects the test mode.  Currently supported\n"
"       modes include 0 (practice only) and 1. Default is test mode.\n"
"    -n num_probs determines the number of problems to be administered.\n"
"       This is considered to be \"infinite\" in practice mode (exit by\n"
"       entering a 'q' instead of an answer).  Default is 100.\n"
"    -r forces a random order presentation, overriding presentation\n"
"       specified in .flash file or default.\n"
"    -t time_max sets the maximum time allowed per problem, in seconds.\n"
"       The variable is ignored in practice mode.  Default is 5.\n"
"    -v selects \"verbose\" operation for debugging, very noisy.\n"
"    -h prints usage statement (this message) and exits.\n"
"\n";

void Usage()
{

 fprintf(stdout,"%s",usage);
 exit(0);

}
