/*
 *========================================================================
 * $Id: present.c 24 2004-01-23 15:20:25Z rgb $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

/*
 *========================================================================
 * Problem presentation routines.  Presentation only -- timer is started
 * elsewhere, and then results are collected and evaluated still elsewhere.
 *========================================================================
 */

#include "flashcard.h"

void present(int pi)
{

 if(verbose){
   wprintw(historyb.win,"Presenting problem %d from problem_list\n",pi);
   wrefresh(historyb.win);
 }

 /*
  * Present the problem.  This needs to be done the FIRST time
  * before forking the timer/grader threads.
  */
 present_problem(pi);

 /*
  * NOW that the problem is presumably heard or on screen the first time,
  * start the timer thread.
  */   
 if(verbose){
   wprintw(historyb.win,"Starting timer thread.\n");
   wrefresh(historyb.win);
 }
 /* if(mode == TEST) pthread_create(&timertid,NULL,(void*)timer,&pi); */
 pthread_create(&timertid,NULL,(void*)timer,&pi);

 /*
  * ...and start the grader thread.  If the grader thread re-presents
  * the problem, it comes out of the timer budget, too bad.
  */
 if(verbose){
   wprintw(historyb.win,"Starting grade_answer() thread.\n");
   wrefresh(historyb.win);
 }
 pthread_create(&probtid,NULL,(void*)grade_answer,&pi);

 /*
  * Set up a "wait" (join) for both threads so that when problem AND timer
  * are gone we can proceed, without leaking threads.
  */
 pthread_join(probtid,NULL);
 pthread_join(timertid,NULL);

 if(verbose){
   wprintw(historyb.win,"Problem %d done with both timer and problem threads.\n",pi);
   wrefresh(historyb.win);
 }

}


int present_problem(int pi)
{

 /*
  * Right now this is a simple redirect into type-specific presentation
  * engines.  Later we may simplify/standardize presentation so that
  * just one presentation call will suffice (this should, in fact, be
  * pretty easy at this point.
  */
 if(verbose){
   wprintw(historyb.win,"present_problem(%d), type %c\n",
     pi,problem_list[pi].type[0]);
   wrefresh(historyb.win);
 }

 /*
  * Is the problem an allowed type?
  */
 if(present_question(pi)){
   switch(problem_list[pi].type[0]){
     case 'A':
       present_prompt(problem_list[pi].question);
       break;
     case 'M':
       present_choices(pi);
     case 'S':
     case 'T':
       present_prompt(problem_list[pi].prompt);
       break;
     default:
       /* Should never get here */
       beep();
       wprintw(historyb.win,"Error2:  Unknown problem type in %s.\n",state.flashfile);
       wrefresh(historyb.win);
       dump_problem(pi,2);
       return(0);
       break;
   }
 }

 /* return success */
 return(1);

}

