/*
 *========================================================================
 * $Id: screen.c 13 2004-01-20 17:10:41Z rgb $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

#include "flashcard.h"


/* 
 *==================================================================
 * Curses/screen routines.
 *==================================================================
 */

/*
 * This routine sets up and starts the curses screen(s).
 */
void start_screen()
{

 int i;

 initscr();	/* Initialize screen for manipulation */
 keypad(stdscr, TRUE);	/* enable keyboard mapping */
 cbreak();      /* character at a time */
 noecho();	/* with no character echo */
 nonl();	/* Don't do NL to NL/CR on output */


 /*
  * Now we need to define a bunch of windows, one for each functional
  * display.  Scrollable windows require two windows, one for the "window"
  * (including the pretty border) and the "pad" that fits inside which
  * displays actual content text.  Pads can be bigger than their viewport
  * windows and can be slid up and down, left and right, within the "virtual"
  * window of their viewport.  Not as intuitive as it could be, but I
  * think we've figured it out.  Now to put it all to work.
  */


 /*
  * Start at the bottom and work our way up to ensure that everything
  * fits.  Leave all the extra space in the history box.
  */

 /* Footer */
 footerb.numrows = 3;
 footerb.numcols = COLS;
 footerb.screenrow = LINES-footerb.numrows;
 footerb.screencol = 0;

 /* Menu */
 menub.numrows = 5;
 menub.numcols = COLS;
 menub.screenrow = footerb.screenrow - menub.numrows + 1;
 menub.screencol = 0;

 /*
  * The scores, percent, and elapsed time form a row pair with their
  * labels
  */
 rightb.numrows = 3;
 rightb.numcols = 13;
 rightb.screenrow = menub.screenrow - rightb.numrows + 1;
 rightb.screencol = 0; 
 rightlb.numrows = 3;
 rightlb.numcols = 13;
 rightlb.screenrow = rightb.screenrow - rightb.numrows + 1;
 rightlb.screencol = 0; 

 wrongb.numrows = 3;
 wrongb.numcols = 13;
 wrongb.screenrow = rightb.screenrow;
 wrongb.screencol = 12; 
 wronglb.numrows = 3;
 wronglb.numcols = 13;
 wronglb.screenrow = rightlb.screenrow;
 wronglb.screencol = 12; 

 pctb.numrows = 3;
 pctb.numcols = 13;
 pctb.screenrow = rightb.screenrow;
 pctb.screencol = 24; 
 pctlb.numrows = 3;
 pctlb.numcols = 13;
 pctlb.screenrow = rightlb.screenrow;
 pctlb.screencol = 24; 

 elapsedb.numrows = 3;
 elapsedb.numcols = 13;
 elapsedb.screenrow = rightb.screenrow;
 elapsedb.screencol = 36; 
 elapsedlb.numrows = 3;
 elapsedlb.numcols = 13;
 elapsedlb.screenrow = rightlb.screenrow;
 elapsedlb.screencol = 36; 

 timerb.numrows = 3;
 timerb.numcols = 13;
 timerb.screenrow = rightb.screenrow;
 timerb.screencol = 48; 
 timerlb.numrows = 3;
 timerlb.numcols = 13;
 timerlb.screenrow = rightlb.screenrow;
 timerlb.screencol = 48; 

 /*
  * answer
  */
 answerb.numrows = 5;
 answerb.numcols = COLS;
 answerb.screenrow = rightlb.screenrow - answerb.numrows + 1;
 answerb.screencol = 0;

 /* History Gets all the way up to line 2 */
 historyb.numrows = answerb.screenrow;
 historyb.numcols = COLS;
 historyb.screenrow = 1;
 historyb.screencol = 0;

 /* Dir exactly overlaps history. */
 dirb.numrows = answerb.screenrow;
 dirb.numcols = COLS;
 dirb.screenrow = 1;
 dirb.screencol = 0;


 /*
  * We need to check to make sure that these will work now.
  */
 if(historyb.numrows < 5 || COLS < 52) {
   fprintf(stderr,"Error:  flashcard needs to be run in a larger xterm\n");
   exit(0);
 }

 /*
  * This is the main flashcard window.  All it holds is the frame border.
  */
 flashwin = newwin(LINES,COLS,0,0);
 wborder(flashwin,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      (char)NULL,(char)NULL,(char)NULL,(char)NULL);
 wrefresh(flashwin);

 /*
  * header.  Header is special and cannot use makebox, so we do it here
  * by hand.
  */
 headerb.numrows = 1;
 headerb.numcols = COLS;
 headerb.screenrow = 0;
 headerb.screencol = 0;
 headerb.win = newwin(headerb.numrows,headerb.numcols,headerb.screenrow,headerb.screencol);
 headerb.pan = new_panel(headerb.win);
 show_panel(headerb.pan);
 scrollok(headerb.win,FALSE);
 /* toplevel menu entries, not very portable */
 mvwprintw(headerb.win,0,1,"File\n");
 mvwprintw(headerb.win,0,9,"Sound\n");
 mvwprintw(headerb.win,0,17,"Mode\n");
 mvwprintw(headerb.win,0,26,"Time\n");
 mvwprintw(headerb.win,0,COLS-6,"Quit\n");
 /* Don't forget to show the windows */
 wrefresh(headerb.win);


 /*
  * We have to do the boxes that don't have mid-frame lines first.
  * history
  */
 makebox(&historyb);
 wborder(historyb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      (char) NULL, (char) NULL,ACS_LTEE,ACS_RTEE );
 wrefresh(historyb.frame);
 /* Put a welcome message at the bottom */
 scrollok(historyb.win,TRUE);
 mvwprintw(historyb.win,historyb.numrows-3,0,"Welcome to Flashcard!\n");
 wrefresh(historyb.win);

 /*
  * Let's TRY to make a window that precisely overlaps the history
  * window.  This will be used only for directory display, maybe.
  */
 makebox(&dirb);
 wborder(dirb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      (char) NULL, (char) NULL,ACS_LTEE,ACS_RTEE );
 wrefresh(dirb.frame);
 /* Put a welcome message at the bottom */
 scrollok(dirb.win,TRUE);
 wrefresh(dirb.win);

 /* Of these two, start by showing the history box on top */
 show_panel(historyb.pan);

 /* menu */
 makebox(&menub);
 wborder(menub.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_LTEE,ACS_RTEE,ACS_LTEE,ACS_RTEE );
 wrefresh(menub.frame);

 /* footer */
 makebox(&footerb);
 wborder(footerb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_LTEE,ACS_RTEE,(char)NULL,(char)NULL );
 wrefresh(footerb.frame);
 mvwprintw(footerb.win,0,0,"             flashcard %d.%.1f     Copyright 2004 Robert G. Brown",VERSION_MAJOR,VERSION_MINOR);
 wrefresh(footerb.win);

 /*
  * Now the boxes with vertical borders inside.
  */

 /* answer */
 makebox(&answerb);
 wborder(answerb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_LTEE,ACS_RTEE,ACS_LTEE,ACS_RTEE );
 wrefresh(answerb.frame);

 /* right, box and label */
 makebox(&rightlb);
 wborder(rightlb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_LTEE,ACS_TTEE,ACS_LTEE,ACS_BTEE );
 wrefresh(rightlb.frame);
 makebox(&rightb);
 wborder(rightb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_LTEE,ACS_PLUS,ACS_LTEE,ACS_BTEE );
 wrefresh(rightb.frame);
 wprintw(rightlb.win,"  Correct");
 wrefresh(rightlb.win);
 /* wrong */
 makebox(&wronglb);
 wborder(wronglb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_TTEE,ACS_TTEE,ACS_LTEE,ACS_BTEE );
 wrefresh(wronglb.frame);
 makebox(&wrongb);
 wborder(wrongb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_PLUS,ACS_PLUS,ACS_BTEE,ACS_BTEE );
 wrefresh(wrongb.frame);
 wprintw(wronglb.win," Incorrect");
 wrefresh(wronglb.win);

 /* pct */
 makebox(&pctlb);
 wborder(pctlb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_TTEE,ACS_TTEE,ACS_LTEE,ACS_BTEE );
 wrefresh(pctlb.frame);
 makebox(&pctb);
 wborder(pctb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_PLUS,ACS_PLUS,ACS_BTEE,ACS_BTEE );
 wrefresh(pctb.frame);
 wprintw(pctlb.win,"  Percent");
 wrefresh(pctlb.win);

 /* elapsed */
 makebox(&elapsedlb);
 wborder(elapsedlb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_TTEE,ACS_TTEE,ACS_LTEE,ACS_BTEE );
 wrefresh(elapsedlb.frame);
 makebox(&elapsedb);
 wborder(elapsedb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_PLUS,ACS_PLUS,ACS_BTEE,ACS_BTEE );
 wrefresh(elapsedb.frame);
 wprintw(elapsedlb.win,"Total  Time");
 wrefresh(elapsedlb.win);

 /* timer */
 makebox(&timerlb);
 wborder(timerlb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_TTEE,ACS_TTEE,ACS_LTEE,ACS_BTEE );
 wrefresh(timerlb.frame);
 makebox(&timerb);
 wborder(timerb.frame,(char)NULL,(char)NULL,(char)NULL,(char)NULL,
      ACS_PLUS,ACS_RTEE,ACS_BTEE,ACS_BTEE );
 wrefresh(timerb.frame);
 wprintw(timerlb.win," Time Left");
 wrefresh(timerlb.win);

}

void start_messagepad()
{

 int i;

 /*
  * Message pad with 100 lines of scrollback
 display.line_messagepad = 1;
 display.lines_messagepad = display.lines_messagewin - 3;
 display.topline_messagepad = 0;
 display.bottomline_messagepad = 100;
 display.showline_messagepad = display.bottomline_messagepad - display.lines_messagepad+1;
 messagepad = newpad(display.bottomline_messagepad,COLS-2);
 keypad(messagepad, TRUE);
 scrollok(messagepad, TRUE);
 for(i=0;i<display.bottomline_messagepad;i++){
   wprintw(messagepad,"Testing %d\n",i);
 }
 prefresh(messagepad,display.showline_messagepad,0,display.line_messagepad,1,display.lines_messagepad-1,COLS-2);

  */
}

/*
 * This routine stops the curses screen if necessary to permit i/o to
 * a normal tty without curses control.  Otherwise it does nothing.
 */
void stop_screen()
{

 endwin();

}

void scroll_messagepad(int scroll_lines)
{

 /*
  * This scrolls messagepad up (+) or down (-) scroll_lines lines or
  * fewer.  We have to make sure that the result stays on the pad, so
  * all this routine does is some bounds checking and then displays the
  * new window onto the pad.

 int new_showline = display.showline_messagepad + scroll_lines;

 if(new_showline < 0) {
   display.showline_messagepad = 0;
 } else if(new_showline > (display.bottomline_messagepad - display.lines_messagepad + 3)){
   display.showline_messagepad = display.bottomline_messagepad - display.lines_messagepad + 3;
 } else {
   display.showline_messagepad = new_showline;
 }

 prefresh(messagepad,display.showline_messagepad,0,display.line_messagepad,1,display.lines_messagepad,COLS-2);

  */
}


void makebox(Box *box)
{

  box->frame = newwin(box->numrows,box->numcols,box->screenrow,box->screencol);
  box->win = newwin(box->numrows-2,box->numcols-2,box->screenrow+1,box->screencol+1);
  box->pan = new_panel(box->win);
  wrefresh(box->frame);
  wrefresh(box->win);
  

}
