/*
 *========================================================================
 * $Id: shuffle.c 25 2004-01-25 05:31:01Z rgb $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

/*
 *========================================================================
 *  Project work template.
 *========================================================================
 */

#include "flashcard.h"

void shufflelist(int *ilist,int imax)
{

 int i,j,tmpi;
 double drand;

 /*
  * Use (unbiased) shuffle algorithm to cover list exactly once
  * and shuffle the contents of ilist (which MAY NOT be sequential
  * indices)!
  */
 if(verbose){
   wprintw(historyb.win,"Shuffling list of %d integers:\n",imax);
   wrefresh(historyb.win);
 }
 for(i=0;i<imax;i++){
   drand = 1.0*rand()/RAND_MAX;
   j = drand*(imax - i) + i;	/* Pick from remaining list */
   if(j != i) {  /* swap only if necessary */
     tmpi = ilist[i];
     ilist[i] = ilist[j];
     ilist[j] = tmpi;
   }
 }

 /*
  * ilist[i] should now be shuffled list of whatever indices were passed
  * in to it.
  */
 j = 0;
 if(verbose){
   wprintw(historyb.win,"Shuffled list:\n");
   for(i=0;i<imax;i++){
     wprintw(historyb.win,"%d %d|",i,ilist[i]);
     j++;
     if((j%10) == 0) wprintw(historyb.win,"\n");
   }
   wprintw(historyb.win,"\n");
   wrefresh(historyb.win);
 }

}


void shuffle()
{

 int i,j;

 if(verbose){
   if(state.presentation == LIST) {
     wprintw(historyb.win,"Problem presentation type is LIST.\n");
   } else {
     wprintw(historyb.win,"Problem presentation type is RANDOM.\n");
   }
   wrefresh(historyb.win);
 }

 for(i=0;i<number_of_problems;i++) iorder[i] = i;
 if(state.presentation == RANDOM){
   /*
    * Feed shufflelist a vector of all indices in order, get back shuffled
    * list.
    */
   shufflelist(iorder,number_of_problems);
 }

 /*
  * iorder[i] should now be shuffled or sequential list of VECTOR site
  * indices as requested.
  */
 j = 0;
 if(verbose){
   wprintw(historyb.win,"Problem presentation order:\n");
   for(i=0;i<number_of_problems;i++){
     wprintw(historyb.win,"%d %d|",i,iorder[i]);
     j++;
     if((j%10) == 0) wprintw(historyb.win,"\n");
   }
   wprintw(historyb.win,"\n");
   wrefresh(historyb.win);
 }

}
