/*
 *========================================================================
 * $Id: snoop_answer.c 30 2004-01-25 22:38:21Z rgb $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

/*
 *========================================================================
 * This is a short ncurses routine that snoops the input queue of
 * the answer window to see if an escaped character has been typed.
 *========================================================================
 */

#include "flashcard.h"

char snoop_answer()
{

 int c1,c2;

 if(verbose){
   wprintw(historyb.win,"snoop_answer(): Entry.\n");
   wrefresh(historyb.win);
 }

 /*
  *=================================================================
  * This is a complicated command.
  *
  * We snoop the queue.  If the first character is 27/Esc, we get the
  * second (the command character) and call options() with that character
  * as the argument.
  *
  * options() might quit, or set state flags to enter e.g. a
  * directory or file selection service.  If it enters something
  * that invalidates the current flashfile, it sets state.suspend
  * to YES.  This in turn signals snoop_answer() to return.
  *
  * snoop_answer is structured so that it will return one of two
  * ways (only).  Either it gets NO Esc-char combo and returns
  * with the queue as it was when it snooped OR it returns (after
  * processing certain commands) in suspended mode.  We MUST
  * therefore check state.suspend in the caller and do the right
  * thing, which should ALWAYS lead to another snoop_answer call,
  * as we might have several Esc-char's in the queue.
  *=================================================================
  */
 noecho();
 c1 = wgetch(answerb.win);
 while(c1 == 27){
   c2 = wgetch(answerb.win);
   options(c2);
   if(state.suspend == YES) {
     echo();
     return(c1);
   }
   /* present_problem(*pi); */
   noecho();
   c1 = wgetch(answerb.win);
 }
 ungetch(c1);
 echo();
 return(c1);

}
